@@ -4,6 +4,7 @@
 # Distributed under the terms of the Modified BSD License.
 
 import sys
+import logging
 
 from traitlets.config import LoggingConfigurable
 
@@ -65,8 +66,8 @@ class CommManager(LoggingConfigurable):
         try:
             return self.comms[comm_id]
         except KeyError:
-            self.log.warn("No such comm: %s", comm_id)
-            if self.log.isEnabledFor(self.log.DEBUG):
+            self.log.warning("No such comm: %s", comm_id)
+            if self.log.isEnabledFor(logging.DEBUG):
                 # don't create the list of keys if debug messages aren't enabled
                 self.log.debug("Current comms: %s", list(self.comms.keys()))
 
@@ -118,7 +119,8 @@ class CommManager(LoggingConfigurable):
         comm = self.get_comm(comm_id)
         if comm is None:
             return
-
+        
+        self.comms[comm_id]._closed = True
         del self.comms[comm_id]
 
         try:
