@@ -167,14 +167,14 @@ class IOPubThread(object):
             return MASTER
         else:
             return CHILD
-    
+
     def start(self):
         """Start the IOPub thread"""
         self.thread.start()
         # make sure we don't prevent process exit
         # I'm not sure why setting daemon=True above isn't enough, but it doesn't appear to be.
         atexit.register(self.stop)
-    
+
     def stop(self):
         """Stop the IOPub thread"""
         if not self.thread.is_alive():
@@ -183,8 +183,10 @@ class IOPubThread(object):
         self.thread.join()
         if hasattr(self._local, 'event_pipe'):
             self._local.event_pipe.close()
-    
+
     def close(self):
+        if self.closed:
+            return
         self.socket.close()
         self.socket = None
 
@@ -206,11 +208,11 @@ class IOPubThread(object):
 
     def send_multipart(self, *args, **kwargs):
         """send_multipart schedules actual zmq send in my thread.
-        
+
         If my thread isn't running (e.g. forked process), send immediately.
         """
         self.schedule(lambda : self._really_send(*args, **kwargs))
-    
+
     def _really_send(self, msg, *args, **kwargs):
         """The callback that actually sends messages"""
         mp_mode = self._check_mp_mode()
@@ -231,10 +233,10 @@ class IOPubThread(object):
 class BackgroundSocket(object):
     """Wrapper around IOPub thread that provides zmq send[_multipart]"""
     io_thread = None
-    
+
     def __init__(self, io_thread):
         self.io_thread = io_thread
-    
+
     def __getattr__(self, attr):
         """Wrap socket attr access for backward-compatibility"""
         if attr.startswith('__') and attr.endswith('__'):
@@ -245,7 +247,7 @@ class BackgroundSocket(object):
                 DeprecationWarning, stacklevel=2)
             return getattr(self.io_thread.socket, attr)
         super(BackgroundSocket, self).__getattr__(attr)
-    
+
     def __setattr__(self, attr, value):
         if attr == 'io_thread' or (attr.startswith('__' and attr.endswith('__'))):
             super(BackgroundSocket, self).__setattr__(attr, value)
@@ -253,7 +255,7 @@ class BackgroundSocket(object):
             warnings.warn("Setting zmq Socket attribute %s on BackgroundSocket" % attr,
                 DeprecationWarning, stacklevel=2)
             setattr(self.io_thread.socket, attr, value)
-    
+
     def send(self, msg, *args, **kwargs):
         return self.send_multipart([msg], *args, **kwargs)
 
@@ -264,7 +266,7 @@ class BackgroundSocket(object):
 
 class OutStream(TextIOBase):
     """A file like object that publishes the stream to a 0MQ PUB socket.
-    
+
     Output is handed off to an IO Thread
     """
 
@@ -276,7 +278,7 @@ class OutStream(TextIOBase):
     topic = None
     encoding = 'UTF-8'
 
-    def __init__(self, session, pub_thread, name, pipe=None):
+    def __init__(self, session, pub_thread, name, pipe=None, echo=None):
         if pipe is not None:
             warnings.warn("pipe argument to OutStream is deprecated and ignored",
                 DeprecationWarning)
@@ -296,6 +298,13 @@ class OutStream(TextIOBase):
         self._flush_pending = False
         self._io_loop = pub_thread.io_loop
         self._new_buffer()
+        self.echo = None
+
+        if echo:
+            if hasattr(echo, 'read') and hasattr(echo, 'write'):
+                self.echo = echo
+            else:
+                raise ValueError("echo argument must be a file like object")
 
     def _is_master_process(self):
         return os.getpid() == self._master_pid
@@ -329,7 +338,7 @@ class OutStream(TextIOBase):
 
         send will happen in the background thread
         """
-        if self.pub_thread.thread.is_alive():
+        if self.pub_thread and self.pub_thread.thread is not None and self.pub_thread.thread.is_alive():
             # request flush on the background thread
             self.pub_thread.schedule(self._flush)
             # wait for flush to actually get through, if we can.
@@ -353,6 +362,15 @@ class OutStream(TextIOBase):
         unless the thread has been destroyed (e.g. forked subprocess).
         """
         self._flush_pending = False
+
+        if self.echo is not None:
+            try:
+                self.echo.flush()
+            except OSError as e:
+                if self.echo is not sys.__stderr__:
+                    print("Flush failed: {}".format(e),
+                          file=sys.__stderr__)
+
         data = self._flush_buffer()
         if data:
             # FIXME: this disables Session's fork-safe check,
@@ -364,6 +382,14 @@ class OutStream(TextIOBase):
                 parent=self.parent_header, ident=self.topic)
 
     def write(self, string):
+        if self.echo is not None:
+            try:
+                self.echo.write(string)
+            except OSError as e:
+                if self.echo is not sys.__stderr__:
+                    print("Write failed: {}".format(e),
+                          file=sys.__stderr__)
+
         if self.pub_thread is None:
             raise ValueError('I/O operation on closed file')
         else:
@@ -390,9 +416,12 @@ class OutStream(TextIOBase):
             for string in sequence:
                 self.write(string)
 
+    def writable(self):
+        return True
+
     def _flush_buffer(self):
         """clear the current buffer and return the current buffer data.
-        
+
         This should only be called in the IO thread.
         """
         data = u''
