@@ -7,6 +7,7 @@ from __future__ import print_function
 
 from datetime import datetime
 from functools import partial
+import itertools
 import logging
 from signal import signal, default_int_handler, SIGINT
 import sys
@@ -44,18 +45,6 @@ CONTROL_PRIORITY = 1
 SHELL_PRIORITY = 10
 ABORT_PRIORITY = 20
 
-class _MessageEvent(tuple):
-    """class for priority message events
-
-    ensures that comparison only invokes the priority entry,
-    not comparing the contents of the messages
-    """
-    def __eq__(self, other):
-        return self[:2] == other[:2]
-
-    def __lt__(self, other):
-        return self[:2] < other[:2]
-
 
 class Kernel(SingletonConfigurable):
 
@@ -124,6 +113,22 @@ class Kernel(SingletonConfigurable):
     # adapt to milliseconds.
     _poll_interval = Float(0.01).tag(config=True)
 
+    stop_on_error_timeout = Float(
+        0.1,
+        config=True,
+        help="""time (in seconds) to wait for messages to arrive
+        when aborting queued requests after an error.
+
+        Requests that arrive within this window after an error
+        will be cancelled.
+
+        Increase in the event of unusually slow network
+        causing significant delays,
+        which can manifest as e.g. "Run all" in a notebook
+        aborting some, but not all, messages after an error.
+        """
+    )
+
     # If the shutdown was requested over the network, we leave here the
     # necessary reply message so it can be sent by our registered atexit
     # handler.  This ensures that the reply is only sent to clients truly at
@@ -200,6 +205,8 @@ class Kernel(SingletonConfigurable):
         sys.stdout.flush()
         sys.stderr.flush()
         self._publish_status(u'idle')
+        # flush to ensure reply is sent
+        self.control_stream.flush(zmq.POLLOUT)
 
     def should_handle(self, stream, msg, idents):
         """Check whether a shell-channel message should be handled
@@ -218,10 +225,6 @@ class Kernel(SingletonConfigurable):
     @gen.coroutine
     def dispatch_shell(self, stream, msg):
         """dispatch shell requests"""
-        # flush control requests first
-        if self.control_stream:
-            self.control_stream.flush()
-
         idents, msg = self.session.feed_identities(msg, copy=False)
         try:
             msg = self.session.deserialize(msg, content=True, copy=False)
@@ -236,6 +239,9 @@ class Kernel(SingletonConfigurable):
         if self._aborting:
             self._send_abort_reply(stream, msg, idents)
             self._publish_status(u'idle')
+            # flush to ensure reply is sent before
+            # handling the next request
+            stream.flush(zmq.POLLOUT)
             return
 
         msg_type = msg['header']['msg_type']
@@ -254,17 +260,26 @@ class Kernel(SingletonConfigurable):
             self.log.warning("Unknown message type: %r", msg_type)
         else:
             self.log.debug("%s: %s", msg_type, msg)
-            self.pre_handler_hook()
+            try:
+                self.pre_handler_hook()
+            except Exception:
+                self.log.debug("Unable to signal in pre_handler_hook:", exc_info=True)
             try:
                 yield gen.maybe_future(handler(stream, idents, msg))
             except Exception:
                 self.log.error("Exception in message handler:", exc_info=True)
             finally:
-                self.post_handler_hook()
+                try:
+                    self.post_handler_hook()
+                except Exception:
+                    self.log.debug("Unable to signal in post_handler_hook:", exc_info=True)
 
         sys.stdout.flush()
         sys.stderr.flush()
         self._publish_status(u'idle')
+        # flush to ensure reply is sent before
+        # handling the next request
+        stream.flush(zmq.POLLOUT)
 
     def pre_handler_hook(self):
         """Hook to execute before calling message handler"""
@@ -280,6 +295,10 @@ class Kernel(SingletonConfigurable):
         self.log.info("Entering eventloop %s", self.eventloop)
         # record handle, so we can check when this changes
         eventloop = self.eventloop
+        if eventloop is None:
+            self.log.info("Exiting as there is no eventloop")
+            return
+
         def advance_eventloop():
             # check if eventloop changed:
             if self.eventloop is not eventloop:
@@ -354,26 +373,39 @@ class Kernel(SingletonConfigurable):
         """
 
         while True:
+            # ensure control stream is flushed before processing shell messages
+            if self.control_stream:
+                self.control_stream.flush()
             # receive the next message and handle it
             try:
                 yield self.process_one()
             except Exception:
                 self.log.exception("Error in message handler")
 
+    _message_counter = Any(
+        help="""Monotonic counter of messages
+
+        Ensures messages of the same priority are handled in arrival order.
+        """,
+    )
+    @default('_message_counter')
+    def _message_counter_default(self):
+        return itertools.count()
+
     def schedule_dispatch(self, priority, dispatch, *args):
         """schedule a message for dispatch"""
-        # via loop.add_callback to ensure everything gets scheduled
-        # on the eventloop
-        self.io_loop.add_callback(
-            lambda: self.msg_queue.put(
-                _MessageEvent((
-                    priority,
-                    self.io_loop.time(),
-                    dispatch,
-                    args,
-                    ))
-                )
+        idx = next(self._message_counter)
+
+        self.msg_queue.put_nowait(
+            (
+                priority,
+                idx,
+                dispatch,
+                args,
+            )
         )
+        # ensure the eventloop wakes up
+        self.io_loop.add_callback(lambda: None)
 
     def start(self):
         """register dispatchers for streams"""
@@ -547,7 +579,7 @@ class Kernel(SingletonConfigurable):
         content = parent['content']
         code = content['code']
         cursor_pos = content['cursor_pos']
-        
+
         matches = yield gen.maybe_future(self.do_complete(code, cursor_pos))
         matches = json_clean(matches)
         completion_msg = self.session.send(stream, 'complete_reply',
@@ -771,10 +803,9 @@ class Kernel(SingletonConfigurable):
     @gen.coroutine
     def _dispatch_abort(self):
         self.log.info("Finishing abort")
-        yield gen.sleep(0.05)
+        yield gen.sleep(self.stop_on_error_timeout)
         self._aborting = False
 
-    @gen.coroutine
     def _send_abort_reply(self, stream, msg, idents):
         """Send a reply to an aborted request"""
         self.log.info("Aborting:")
@@ -859,7 +890,7 @@ class Kernel(SingletonConfigurable):
                 self.log.warning("Invalid Message:", exc_info=True)
             except KeyboardInterrupt:
                 # re-raise KeyboardInterrupt, to truncate traceback
-                raise KeyboardInterrupt
+                raise KeyboardInterrupt("Interrupted by user") from None
             else:
                 break
         try:
